@@ -1,6 +1,6 @@
 //! This is special benchmark, to measure empty queue iteration overhead.
 
-use rc_event_queue::mpmc::{EventQueue, Settings};
+use rc_event_queue::mpmc::{CleanupMode, EventQueue, LendingIterator, Settings};
 use criterion::{black_box, criterion_group, criterion_main, Criterion, BenchmarkId};
 use std::time::{Instant, Duration};
 use std::collections::VecDeque;
@@ -9,7 +9,7 @@ struct EventQueueSettings{}
 impl Settings for EventQueueSettings{
     const MIN_CHUNK_SIZE: u32 = 512;
     const MAX_CHUNK_SIZE: u32 = 512;
-    const AUTO_CLEANUP: bool = false;
+    const CLEANUP: CleanupMode = CleanupMode::Never;
 }
 
 fn bench_event_reader(iters: u64) -> Duration{
@@ -18,7 +18,7 @@ fn bench_event_reader(iters: u64) -> Duration{
         let event = EventQueue::<usize, EventQueueSettings>::new();
         let mut reader = event.subscribe();
         let start = Instant::now();
-        for i in reader.iter(){
+        while let Some(i) = reader.iter().next(){
             black_box(i);
         }
         total += start.elapsed();
